%define base 2	; base note, matches first SFX
org 100h

push   0xa000	; + 2 bytes for 
pop    es		; perfect alignment
mov    al,0x13	; mode13h = 320*200 pixels
; palette code from "ocean" effect, nice blue shades
L:	; assume low red component (DosBox)
	int 0x10		; used twice (set mode, set col)
	dec ch			; green channel
	mov ax, 0x1010  ; col change subfunction
	dec	bx			; color number
	jnz L

mov ax,0x251c ; call back for music
mov dl,M ; dosbox only!! otherwise DX
int 0x21

mov bp,2302 ; starting frame number

push si			; save SI
mov dx,0x330	; output all the data
mov si,musi		; at @musi to the MIDI port
mov cl,30		; assume activated UART mode
rep outsb		; send ALL the data!
pop si			; restore SI

F: 				; basically mentors engine from here on
mul bx			; tough to shrink this one in size!
const:
inc ax			; hidden 320 here 0x01 0x40
add ax,di
div word [byte si+const-0x100] ; divide scr ptr by 320
mov cx,127			; magic constant 
sub dx,cx			; DX = Y ; AX = x
pusha  	; -4 AX, -6 CX, -8 DX, -10 BX, -12 SP, -14 BP, -16 SI ...
	mov cl,4 ; repeat four times			
	LL:
		mov si,-8			; reset SI to calculate coefficents
		fild word [si-6]	; BP ... -14 BP = framecount
		fidiv word [si-8]	; BP/SI ... -16 SI = 0x100
		fsincos 			; cos(a) sin(a)
		fimul word [bx+si]	; cos(a)*y sin(a)... -8 DX
		xor bl,2
		fxch st1			; sin(a) cos(a)*y
		fimul word [bx+si]	; sin(a)*CX cos(a)*y -6 CX
		jnz Y
		fchs   ;			; -sin(a)*CX cos(a)*y
		mov word [si],ax	; DX->AX(!) on second and fourth run
		Y:
		faddp st1,st0		; cos(a)*y-sin(a)*CX
		lodsw  				; just inc SI twice
		mov ax,word [bx+si]
		fistp word [bx+si]	; store
	loop LL	; rinse and repeat
	E:
	mov cl,1	; start for menger constant
	B:
	xor bp,bp	; reset hit counter for geometry
	mov si,-8	; reset SI to use calculated coefficents
	A:
	lodsw; load calculated coefficents and advance

	mul bx
	add ax,word [si-11]

	mul cx
	add ax,0x19ab		; distorted constants here
	cmp ax,0x7777		; for different geometry
	sbb bp,byte 0		; 2 of 3 are enough for hit
	jnp Z				; abort
	cmp si,byte -2 		; repeat three times (-6,-4,-2)
	jl A
	imul cx,cx,byte 3	; 3^n menger constant
	test cl,cl			; up to 81
	jns B				; then quit (243>128, sign flag)
	jmp short D
	Z:
	inc bl				; add bl,2 for double speed
	jne E				
	dec bx	; palette overflow at the back otherwise
	D:
	shrd ax,bx,18 ; get scaled color value in AX 
	test di,di	; subtle texture
	jnp ppp		; on every other
	dec ax		; pixel
	ppp:
	stosb		; finally, output pixel
popa
dec	di				; pseudo randomizing
imul di,byte 117	; the screen position
test di,di			; all pixels done?
jne F				; if not repeat
inc bp				; advance frame counter
inc bp				; by two (speed up)
test bp, 32			; enough of this scene?
jz FG				; no ...
add bp, 224			; yes, advance to next scene
FG:
in al,0x60			; get key status
dec al				; ESC?
jnz F				; if not, repeat
RRR:ret				; if yes, quit
M:
pusha
	test byte [fs:0x46c],7	; play notes on every 8th timer tick
	jnz Q
	mov dx,0x330	; MIDI port adress
	mov cl,6		; we will play 6 notes
	LLL:
		mov al,0x94		; play note on channel 4
		out dx,al
		push dx			; get some random number
			rdtsc
			xor al,ah
		pop dx			; (and protect DX)
		aam 5			; pentatonic reduction ...
		mov bl,al		; and look up
		mov bl,[bx+musi]; .. of stored notes

		mov al,ah		; using the rest of RNG	
		aam 7			; to pitch by a maximum of 7
		imul ax,byte 12	; octaves (12)
		add al,bl
		out dx,al		; play this note
		mov al,80		; set volume to 80
		out dx,al		; set the volume
	loop LLL			; rinse and repeat
Q:
popa					; restore stack
iret					; back to main program
musi:					; note data, just a pentatonic patch
db 10+base,13+base,15+base,18+base,20+base
musi2:
db 0xc4,4  ; set instrument on channel 4 to e-piano
db 0xc1,60 ; set instrument on channel 1 to french horn
db 0x91,29,127,29,113,29+12,100 ; play LOTS of bass notes
; set drumkit to SFX and play rain, bubbles, train, spaceship, etc.
db 0xc9,56,0x99,79,63,84,70,71,127,62,17,68,57